
package edu.uthscsa.ric.plugins.mangoplugin;

import java.io.File;
import java.net.URISyntaxException;
import java.net.URL;

import edu.uthscsa.ric.cli.ConsolePlugin;
import edu.uthscsa.ric.mango.MangoContext;
import edu.uthscsa.ric.mango.MangoData;
import edu.uthscsa.ric.mango.MangoPlugin;
import edu.uthscsa.ric.mango.ViewerController;
import edu.uthscsa.ric.mango.viewerslice.VolumeManager;
import edu.uthscsa.ric.volume.ImageVolume;
import edu.uthscsa.ric.volume.InvalidHeaderException;


/**
 * This plugin will add one to every value in the image -- both as a plugin and at the command-line.
 * 
 * @author michaelmartinez
 * 
 */
public class ExampleCliPlugin implements ConsolePlugin, MangoPlugin {

	@Override
	public Object doOperation(MangoContext context, String... args) {
		if (args.length < 1) {
			System.err.println(getUsage());
			return null;
		}

		final ImageVolume iv = context.makeNewVolume();
		try {
			final File infile = new File(args[0]);

			iv.readFiles(infile.toURI());
			doOperation(iv);

			String filename = infile.getName();
			if (filename.indexOf(".") != -1) {
				filename = filename.substring(0, filename.indexOf("."));
			}

			filename += "_new.nii.gz";

			final File outfile = new File(infile.getParent(), filename);

			iv.writeFilesAs(outfile, ImageVolume.HEADER_FORMAT_NIFTI, iv.getImageType(), null, false, false);
		} catch (final InvalidHeaderException e) {
			e.printStackTrace();
		} catch (final URISyntaxException e) {
			e.printStackTrace();
		} catch (final Exception e) {
			e.printStackTrace();
		}

		return null;
	}



	@Override
	public void doOperation(MangoContext mango, VolumeManager viewer) {
		final ImageVolume iv = viewer.getBaseVolume();
		doOperation(iv);
		viewer.updateViewer();
	}



	@Deprecated
	@Override
	public void doOperation(final MangoData data, final ViewerController controller) {
		//		final ScreenImage baseScreenImage = data.getBaseScreenImage();
		//		final ImageVolume iv = baseScreenImage.getImageVolume();
		//		doOperation(iv);
		//		controller.pluginOperationCompleted();
	}



	@Override
	public String getCommandLineName() {
		return "addoneto";
	}



	@Override
	public String getMinimumVersionSupported() {
		return null;
	}



	@Override
	public String getPluginName() {
		return "Add one to every voxel (cli)";
	}



	@Override
	public URL getPluginURL() {
		return null;
	}



	@Override
	public String getUsage() {
		return "addoneto <file>";
	}



	@Override
	public String getVersion() {
		return null;
	}



	@Override
	public boolean hasNewerVersion() {
		return false;
	}



	private void doOperation(final ImageVolume iv) {
		final long numberOfVoxels = iv.getImageDimensions().getNumVoxelsTimeseries();
		double value = 0;

		for (int ctr = 0; ctr < numberOfVoxels; ctr++) {
			value = iv.getVoxelValueForOffset(ctr);
			iv.putVoxelValueAtOffset(ctr, value + 1);
		}
	}
}
