
#include <jni.h>
#include <string>

#ifndef MJNI_H
#define MJNI_H


// Returns a pointer to the native image data array
float* getImageData(JNIEnv* env, jobject imageVol) {
	jclass cls = (env)->GetObjectClass(imageVol);
	jmethodID mid = (env)->GetMethodID(cls, "getWorkBuffer", "()Ljava/nio/ByteBuffer;");
	jobject buf = (env)->CallObjectMethod(imageVol, mid);
	return (float*)((env)->GetDirectBufferAddress(buf));
}



// (deprecated) Returns a pointer to the base image data array
float* getBaseImageData(JNIEnv* env, jobject data) {
	jclass cls = (env)->GetObjectClass(data);
	jmethodID mid = (env)->GetMethodID(cls, "getBaseImage", "()Ledu/uthscsa/ric/mango/MangoImage;");
		
	jobject baseImage = (env)->CallObjectMethod(data, mid);
	
	cls = (env)->GetObjectClass(baseImage);
	mid = (env)->GetMethodID(cls, "getImageVolume", "()Ledu/uthscsa/ric/volume/ImageVolume;");
		
	jobject imageVol = (env)->CallObjectMethod(baseImage, mid);
	
	cls = (env)->GetObjectClass(imageVol);
	mid = (env)->GetMethodID(cls, "getWorkBuffer", "()Ljava/nio/ByteBuffer;");
			
	jobject buf = (env)->CallObjectMethod(imageVol, mid);

	return (float*)((env)->GetDirectBufferAddress(buf));
}



// (deprecated) Makes a new image, with the given name, that has the same parameters as the base image
float* makeNewImageWithSameDimensions(JNIEnv* env, jobject controller, jobject data, char* name) {
	jstring jname = (env)->NewStringUTF(name);
 
	jclass cls = (env)->GetObjectClass(data);
	jmethodID mid = (env)->GetMethodID(cls, "getBaseImage", "()Ledu/uthscsa/ric/mango/MangoImage;");
		
	jobject baseImage = (env)->CallObjectMethod(data, mid);
	
	cls = (env)->GetObjectClass(baseImage);
	mid = (env)->GetMethodID(cls, "getImageDimensions", "()Ledu/uthscsa/ric/volume/ImageDimensions;");
	
	jobject imageDims = (env)->CallObjectMethod(baseImage, mid);
	
	cls = (env)->GetObjectClass(baseImage);
	mid = (env)->GetMethodID(cls, "getVoxelDimensions", "()Ledu/uthscsa/ric/volume/VoxelDimensions;");	
		
	jobject voxelDims = (env)->CallObjectMethod(baseImage, mid);	

	cls = (env)->GetObjectClass(baseImage);
	mid = (env)->GetMethodID(cls, "getImageVolume", "()Ledu/uthscsa/ric/volume/ImageVolume;");
		
	jobject imageVolOriginal = (env)->CallObjectMethod(baseImage, mid);	
	
	cls = (env)->GetObjectClass(imageVolOriginal);
	mid = (env)->GetMethodID(cls, "getOrientationString", "()Ljava/lang/String;");
		
	jstring orientation = (jstring)(env)->CallObjectMethod(imageVolOriginal, mid);		
	
	cls = (env)->GetObjectClass(controller);
	mid = (env)->GetMethodID(cls, "addNewImage", "(Ledu/uthscsa/ric/volume/ImageDimensions;Ledu/uthscsa/ric/volume/VoxelDimensions;Ljava/lang/String;Ljava/lang/String;)Ledu/uthscsa/ric/mango/MangoImage;");	
		
	jobject dataNew = (env)->CallObjectMethod(controller, mid, imageDims, voxelDims, orientation, jname);	

	cls = (env)->GetObjectClass(dataNew);
	mid = (env)->GetMethodID(cls, "getImageVolume", "()Ledu/uthscsa/ric/volume/ImageVolume;");
				
	jobject imageVol = (env)->CallObjectMethod(dataNew, mid);
	
	cls = (env)->GetObjectClass(imageVol);
	mid = (env)->GetMethodID(cls, "getWorkBuffer", "()Ljava/nio/ByteBuffer;");
					
	jobject buf = (env)->CallObjectMethod(imageVol, mid);

	return (float*)((env)->GetDirectBufferAddress(buf));
}



// (deprecated) Gets the base image dimensions
void getBaseImageDims(JNIEnv* env, jobject data, int& cols, int& rows, int& slices, int& timepoints) {
	jclass cls = (env)->GetObjectClass(data);
	jmethodID mid = (env)->GetMethodID(cls, "getBaseImage", "()Ledu/uthscsa/ric/mango/MangoImage;");
		
	jobject baseImage = (env)->CallObjectMethod(data, mid);
	
	cls = (env)->GetObjectClass(baseImage);
	mid = (env)->GetMethodID(cls, "getImageDimensions", "()Ledu/uthscsa/ric/volume/ImageDimensions;");
	
	jobject imageDims = (env)->CallObjectMethod(baseImage, mid);

	cls = (env)->GetObjectClass(imageDims);
	
	mid = (env)->GetMethodID(cls, "getCols", "()I");
	cols = (int)(env)->CallIntMethod(imageDims, mid);

	mid = (env)->GetMethodID(cls, "getRows", "()I");
	rows = (int)(env)->CallIntMethod(imageDims, mid);	
	
	mid = (env)->GetMethodID(cls, "getSlices", "()I");
	slices = (int)(env)->CallIntMethod(imageDims, mid);	

	mid = (env)->GetMethodID(cls, "getTimepoints", "()I");
	timepoints = (int)(env)->CallIntMethod(imageDims, mid);
}



// (deprecated) Gets the base image voxel dimensions
void getBaseVoxelDims(JNIEnv* env, jobject data, double& colSize, double& rowSize, double& sliceThickness, double& tr) {
	jclass cls = (env)->GetObjectClass(data);
	jmethodID mid = (env)->GetMethodID(cls, "getBaseImage", "()Ledu/uthscsa/ric/mango/MangoImage;");
		
	jobject baseImage = (env)->CallObjectMethod(data, mid);
	
	cls = (env)->GetObjectClass(baseImage);
	mid = (env)->GetMethodID(cls, "getVoxelDimensions", "()Ledu/uthscsa/ric/volume/VoxelDimensions;");
	
	jobject imageDims = (env)->CallObjectMethod(baseImage, mid);

	cls = (env)->GetObjectClass(imageDims);
	
	mid = (env)->GetMethodID(cls, "getColSize", "(Z)D");
	colSize = (double)(env)->CallDoubleMethod(imageDims, mid, true);

	mid = (env)->GetMethodID(cls, "getRowSize", "(Z)D");
	rowSize = (double)(env)->CallDoubleMethod(imageDims, mid, true);	
	
	mid = (env)->GetMethodID(cls, "getSliceThickness", "(Z)D");
	sliceThickness = (double)(env)->CallDoubleMethod(imageDims, mid, true);	

	mid = (env)->GetMethodID(cls, "getTR", "()D");
	tr = (double)(env)->CallDoubleMethod(imageDims, mid);
}



// (deprecated) Gets the base image data scale slopes
void getDataScaleSlopes(JNIEnv* env, jobject data, float* slopes) {
	jclass cls = (env)->GetObjectClass(data);
	jmethodID mid = (env)->GetMethodID(cls, "getBaseImage", "()Ledu/uthscsa/ric/mango/MangoImage;");
		
	jobject baseImage = (env)->CallObjectMethod(data, mid);

	cls = (env)->GetObjectClass(baseImage);
	mid = (env)->GetMethodID(cls, "getImageRange", "()Ledu/uthscsa/ric/volume/ImageRange;");
		
	jobject imageRange = (env)->CallObjectMethod(baseImage, mid);	
	
	cls = (env)->GetObjectClass(imageRange);
	mid = (env)->GetMethodID(cls, "getDataScaleSlopes", "()[F");

	jfloatArray arr = (jfloatArray)(env)->CallObjectMethod(imageRange, mid);
	jsize len = (env)->GetArrayLength(arr);

	jfloat* body = (env)->GetFloatArrayElements(arr, 0);

	for(int ctr = 0; ctr < len; ctr++)
		slopes[ctr] = body[ctr];

	(env)->ReleaseFloatArrayElements(arr, body, 0);
}



// (deprecated) Gets the base image data scale intercepts
void getDataScaleIntercepts(JNIEnv* env, jobject data, float* intercepts) {
	jclass cls = (env)->GetObjectClass(data);
	jmethodID mid = (env)->GetMethodID(cls, "getBaseImage", "()Ledu/uthscsa/ric/mango/MangoImage;");
		
	jobject baseImage = (env)->CallObjectMethod(data, mid);

	cls = (env)->GetObjectClass(baseImage);
	mid = (env)->GetMethodID(cls, "getImageRange", "()Ledu/uthscsa/ric/volume/ImageRange;");
		
	jobject imageRange = (env)->CallObjectMethod(baseImage, mid);	
	
	cls = (env)->GetObjectClass(imageRange);
	mid = (env)->GetMethodID(cls, "getDataScaleIntercepts", "()[F");

	jfloatArray arr = (jfloatArray)(env)->CallObjectMethod(imageRange, mid);
	jsize len = (env)->GetArrayLength(arr);

	jfloat* body = (env)->GetFloatArrayElements(arr, 0);

	for(int ctr = 0; ctr < len; ctr++)
		intercepts[ctr] = body[ctr];

	(env)->ReleaseFloatArrayElements(arr, body, 0);
}



// (deprecated) Gets the orientation of the base image as a six character string (e.g., XYZ+--)
void getOrientationString(JNIEnv* env, jobject data, std::string& orientation) {
 
	jclass cls = (env)->GetObjectClass(data);
	jmethodID mid = (env)->GetMethodID(cls, "getBaseImage", "()Ledu/uthscsa/ric/mango/MangoImage;");
		
	jobject baseImage = (env)->CallObjectMethod(data, mid);

	cls = (env)->GetObjectClass(baseImage);
	mid = (env)->GetMethodID(cls, "getImageVolume", "()Ledu/uthscsa/ric/volume/ImageVolume;");
		
	jobject imageVol = (env)->CallObjectMethod(baseImage, mid);	
	
	cls = (env)->GetObjectClass(imageVol);
	mid = (env)->GetMethodID(cls, "getOrientationString", "()Ljava/lang/String;");	

	jstring javaString = (jstring)(env)->CallObjectMethod(imageVol, mid);	
	const char* nativeString = (env)->GetStringUTFChars(javaString, 0);

	orientation.assign(nativeString);

	(env)->ReleaseStringUTFChars(javaString, nativeString);
}

#endif
