
package edu.uthscsa.ric.plugins.mangoplugin;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.net.URL;

import edu.uthscsa.ric.mango.MangoContext;
import edu.uthscsa.ric.mango.MangoData;
import edu.uthscsa.ric.mango.MangoPlugin;
import edu.uthscsa.ric.mango.ViewerController;
import edu.uthscsa.ric.mango.viewerslice.VolumeManager;
import edu.uthscsa.ric.volume.ImageDimensions;
import edu.uthscsa.ric.volume.ImageVolume;


/**
 * This plugin will add one to every value in the image, performing the operation in a C++ library. Note, the included makefile is currently only configured for
 * OS X.
 * 
 * @author michaelmartinez
 * 
 */
public class MangoPluginNative implements MangoPlugin {

	public static final boolean LINUX = (System.getProperty("os.name", "").toUpperCase().indexOf("LINUX") != -1);
	public static final boolean MAC = (System.getProperty("os.name", "").toUpperCase().indexOf("MAC OS") != -1);
	public static final boolean WINDOWS = (System.getProperty("os.name", "").toUpperCase().indexOf("WINDOWS") != -1);
	public static final String LIB_FILE_NAME;
	public static final String LIB_NAME = "MangoPluginNative";
	public static final String PLUGIN_NAME = "Add One To Every Voxel (Native)";
	private static boolean libraryLoaded;

	static {
		if (MAC) {
			LIB_FILE_NAME = "lib" + LIB_NAME + ".jnilib";
		} else if (WINDOWS) {
			LIB_FILE_NAME = LIB_NAME + ".dll";
		} else {
			LIB_FILE_NAME = "lib" + LIB_NAME + ".so";
		}
	}



	@Override
	public void doOperation(MangoContext mango, VolumeManager viewer) {
		if (!libraryLoaded) {
			loadLibrary(mango);
			libraryLoaded = true;
		}

		ImageVolume baseVol = viewer.getBaseVolume();
		ImageDimensions id = baseVol.getImageDimensions();
		doOperationNative(baseVol, id.getCols(), id.getRows(), id.getSlices(), id.getTimepoints());
		viewer.updateViewer();
	}



	@Deprecated
	@Override
	public void doOperation(MangoData data, ViewerController controller) {
		//		if (!libraryLoaded) {
		//			loadLibrary(controller);
		//			libraryLoaded = true;
		//		}
		//
		//		doOperationNative(data, controller);
		//		controller.pluginOperationCompleted();
	}



	@Override
	public String getMinimumVersionSupported() {
		return null;
	}



	@Override
	public String getPluginName() {
		return PLUGIN_NAME;
	}



	@Override
	public URL getPluginURL() {
		return null;
	}



	@Override
	public String getVersion() {
		return null;
	}



	@Override
	public boolean hasNewerVersion() {
		return false;
	}



	native int doOperationNative(Object imageVol, int cols, int rows, int slices, int timepoints);



	private void loadLibrary(MangoContext mango) {
		File libDir = mango.getLibDir();
		BufferedInputStream fileRead = null;
		BufferedOutputStream fileWrite = null;

		try {
			File standardFile = new File(libDir, LIB_FILE_NAME);

			fileRead = new BufferedInputStream(this.getClass().getResourceAsStream(LIB_FILE_NAME));
			fileWrite = new BufferedOutputStream(new FileOutputStream(standardFile));
			byte[] buffer = new byte[2048];

			int bytesRead = fileRead.read(buffer);

			while (bytesRead != -1) {
				fileWrite.write(buffer, 0, bytesRead);
				bytesRead = fileRead.read(buffer);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		} finally {
			try {
				fileRead.close();
			} catch (Exception ex) {}

			try {
				fileWrite.close();
			} catch (Exception ex) {}
		}

		System.loadLibrary(LIB_NAME);
	}
}
