
package edu.uthscsa.ric.plugins.mangoplugin;

import java.net.URL;

import edu.uthscsa.ric.mango.MangoContext;
import edu.uthscsa.ric.mango.MangoData;
import edu.uthscsa.ric.mango.MangoPlugin;
import edu.uthscsa.ric.mango.ProgressMeter;
import edu.uthscsa.ric.mango.ViewerController;
import edu.uthscsa.ric.mango.viewerslice.VolumeManager;
import edu.uthscsa.ric.roi.ROIData;
import edu.uthscsa.ric.volume.ImageVolume;


/**
 * This plugin will create a dynamic red ROI mask series using the display minimum as a threshold.
 * 
 * @author michaelmartinez
 * 
 */
public class ExampleROISeriesPlugin implements MangoPlugin {

	@Override
	public void doOperation(final MangoContext mango, final VolumeManager viewer) {
		Thread workThread = new Thread(new Runnable() {

			@Override
			public void run() {
				doThreshold(viewer, mango);
			}
		});

		workThread.start();
	}



	@Deprecated
	@Override
	public void doOperation(final MangoData data, final ViewerController controller) {
		//		Thread workThread = new Thread(new Runnable() {
		//
		//			@Override
		//			public void run() {
		//				doThreshold(data, controller);
		//			}
		//		});
		//
		//		workThread.start();
	}



	@Override
	public String getMinimumVersionSupported() {
		return null;
	}



	@Override
	public String getPluginName() {
		return "Create dynamic red ROI series from threshold";
	}



	@Override
	public URL getPluginURL() {
		return null;
	}



	@Override
	public String getVersion() {
		return null;
	}



	@Override
	public boolean hasNewerVersion() {
		return false;
	}



	private void doThreshold(VolumeManager viewer, MangoContext mango) {
		ROIData roiData = viewer.getRoiData();
		ImageVolume iv = viewer.getBaseVolume();
		double threshold = viewer.getVolumeDisplayRangeMin(iv);

		int dimX = iv.getImageDimensions().getX();
		int dimY = iv.getImageDimensions().getY();
		int dimZ = iv.getImageDimensions().getZ();
		int seriesPoints = iv.getImageDimensions().getTimepoints();

		if (seriesPoints > 1) {
			roiData.setUseROISeries(true);
		}

		roiData.initMask(ROIData.MASK_RED);

		ProgressMeter progress = mango.makeProgressMeter();
		progress.setDescription("Threshold");
		progress.start(0, 0, seriesPoints);

		for (int ctrT = 0; ctrT < seriesPoints; ctrT++) {
			progress.setValue(ctrT);

			for (int ctrZ = 0; ctrZ < dimZ; ctrZ++) {
				for (int ctrY = 0; ctrY < dimY; ctrY++) {
					for (int ctrX = 0; ctrX < dimX; ctrX++) {
						double value = iv.getVoxelValueAtIndex(ctrX, ctrY, ctrZ, ctrT);
						if (value > threshold) {
							roiData.addMaskValue(ctrX, ctrY, ctrZ, ctrT, ROIData.MASK_RED);
						}
					}
				}
			}
		}

		progress.setValue(progress.getMax());
		roiData.roiMaskOperationCompleted();
		viewer.updateViewer();
	}
}
