
package edu.uthscsa.ric.plugins.mangoplugin;

import java.net.URL;

import edu.uthscsa.ric.mango.MangoContext;
import edu.uthscsa.ric.mango.MangoData;
import edu.uthscsa.ric.mango.MangoPlugin;
import edu.uthscsa.ric.mango.ViewerController;
import edu.uthscsa.ric.mango.viewerslice.VolumeManager;
import edu.uthscsa.ric.roi.ROIData;
import edu.uthscsa.ric.volume.ImageVolume;


/**
 * This plugin will create a red ROI mask using the display minimum as a threshold.
 * 
 * @author michaelmartinez
 * 
 */
public class ExampleROIPlugin implements MangoPlugin {

	@Override
	public void doOperation(MangoContext mango, VolumeManager viewer) {
		ROIData roiData = viewer.getRoiData();
		roiData.initMask(ROIData.MASK_RED);

		ImageVolume iv = viewer.getBaseVolume();
		double threshold = viewer.getVolumeDisplayRangeMin(iv);

		int dimX = iv.getImageDimensions().getX();
		int dimY = iv.getImageDimensions().getY();
		int dimZ = iv.getImageDimensions().getZ();

		for (int ctrZ = 0; ctrZ < dimZ; ctrZ++) {
			for (int ctrY = 0; ctrY < dimY; ctrY++) {
				for (int ctrX = 0; ctrX < dimX; ctrX++) {
					double value = iv.getVoxelValueAtIndex(ctrX, ctrY, ctrZ);
					if (value > threshold) {
						roiData.addMaskValue(ctrX, ctrY, ctrZ, ROIData.MASK_RED);
					}
				}
			}
		}

		roiData.roiMaskOperationCompleted();
		viewer.updateViewer();
	}



	@Deprecated
	@Override
	public void doOperation(MangoData data, ViewerController controller) {
		//		ROIData roiData = data.getROIData();
		//		roiData.initMask(ROIData.MASK_RED);
		//
		//		ScreenImage baseScreenImage = data.getBaseScreenImage();
		//		double threshold = baseScreenImage.getScreenMin();
		//
		//		ImageVolume iv = baseScreenImage.getImageVolume();
		//		int dimX = iv.getImageDimensions().getX();
		//		int dimY = iv.getImageDimensions().getY();
		//		int dimZ = iv.getImageDimensions().getZ();
		//
		//		for (int ctrZ = 0; ctrZ < dimZ; ctrZ++) {
		//			for (int ctrY = 0; ctrY < dimY; ctrY++) {
		//				for (int ctrX = 0; ctrX < dimX; ctrX++) {
		//					double value = iv.getVoxelValueAtIndex(ctrX, ctrY, ctrZ);
		//					if (value > threshold) {
		//						roiData.addMaskValue(ctrX, ctrY, ctrZ, ROIData.MASK_RED);
		//					}
		//				}
		//			}
		//		}
		//
		//		roiData.roiMaskOperationCompleted();
		//		controller.pluginOperationCompleted();
	}



	@Override
	public String getMinimumVersionSupported() {
		return null;
	}



	@Override
	public String getPluginName() {
		return "Create red ROI from threshold";
	}



	@Override
	public URL getPluginURL() {
		return null;
	}



	@Override
	public String getVersion() {
		return null;
	}



	@Override
	public boolean hasNewerVersion() {
		return false;
	}
}
